/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiCalendarDateReference
    /// </summary>
    [DataContract]
    public partial class EdFiCalendarDateReference :  IEquatable<EdFiCalendarDateReference>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiCalendarDateReference" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiCalendarDateReference() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiCalendarDateReference" /> class.
        /// </summary>
        /// <param name="calendarCode">The identifier for the Calendar. (required).</param>
        /// <param name="date">The month, day, and year of the CalendarEvent. (required).</param>
        /// <param name="schoolId">The identifier assigned to a school. (required).</param>
        /// <param name="schoolYear">The identifier for the school year associated with the Calendar. (required).</param>
        /// <param name="link">link.</param>
        public EdFiCalendarDateReference(string calendarCode = default(string), DateTime? date = default(DateTime?), int? schoolId = default(int?), int? schoolYear = default(int?), Link link = default(Link))
        {
            // to ensure "calendarCode" is required (not null)
            if (calendarCode == null)
            {
                throw new InvalidDataException("calendarCode is a required property for EdFiCalendarDateReference and cannot be null");
            }
            else
            {
                this.CalendarCode = calendarCode;
            }
            // to ensure "date" is required (not null)
            if (date == null)
            {
                throw new InvalidDataException("date is a required property for EdFiCalendarDateReference and cannot be null");
            }
            else
            {
                this.Date = date;
            }
            // to ensure "schoolId" is required (not null)
            if (schoolId == null)
            {
                throw new InvalidDataException("schoolId is a required property for EdFiCalendarDateReference and cannot be null");
            }
            else
            {
                this.SchoolId = schoolId;
            }
            // to ensure "schoolYear" is required (not null)
            if (schoolYear == null)
            {
                throw new InvalidDataException("schoolYear is a required property for EdFiCalendarDateReference and cannot be null");
            }
            else
            {
                this.SchoolYear = schoolYear;
            }
            this.Link = link;
        }
        
        /// <summary>
        /// The identifier for the Calendar.
        /// </summary>
        /// <value>The identifier for the Calendar.</value>
        [DataMember(Name="calendarCode", EmitDefaultValue=false)]
        public string CalendarCode { get; set; }

        /// <summary>
        /// The month, day, and year of the CalendarEvent.
        /// </summary>
        /// <value>The month, day, and year of the CalendarEvent.</value>
        [DataMember(Name="date", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? Date { get; set; }

        /// <summary>
        /// The identifier assigned to a school.
        /// </summary>
        /// <value>The identifier assigned to a school.</value>
        [DataMember(Name="schoolId", EmitDefaultValue=false)]
        public int? SchoolId { get; set; }

        /// <summary>
        /// The identifier for the school year associated with the Calendar.
        /// </summary>
        /// <value>The identifier for the school year associated with the Calendar.</value>
        [DataMember(Name="schoolYear", EmitDefaultValue=false)]
        public int? SchoolYear { get; set; }

        /// <summary>
        /// Gets or Sets Link
        /// </summary>
        [DataMember(Name="link", EmitDefaultValue=false)]
        public Link Link { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiCalendarDateReference {\n");
            sb.Append("  CalendarCode: ").Append(CalendarCode).Append("\n");
            sb.Append("  Date: ").Append(Date).Append("\n");
            sb.Append("  SchoolId: ").Append(SchoolId).Append("\n");
            sb.Append("  SchoolYear: ").Append(SchoolYear).Append("\n");
            sb.Append("  Link: ").Append(Link).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiCalendarDateReference);
        }

        /// <summary>
        /// Returns true if EdFiCalendarDateReference instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiCalendarDateReference to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiCalendarDateReference input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.CalendarCode == input.CalendarCode ||
                    (this.CalendarCode != null &&
                    this.CalendarCode.Equals(input.CalendarCode))
                ) && 
                (
                    this.Date == input.Date ||
                    (this.Date != null &&
                    this.Date.Equals(input.Date))
                ) && 
                (
                    this.SchoolId == input.SchoolId ||
                    (this.SchoolId != null &&
                    this.SchoolId.Equals(input.SchoolId))
                ) && 
                (
                    this.SchoolYear == input.SchoolYear ||
                    (this.SchoolYear != null &&
                    this.SchoolYear.Equals(input.SchoolYear))
                ) && 
                (
                    this.Link == input.Link ||
                    (this.Link != null &&
                    this.Link.Equals(input.Link))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.CalendarCode != null)
                    hashCode = hashCode * 59 + this.CalendarCode.GetHashCode();
                if (this.Date != null)
                    hashCode = hashCode * 59 + this.Date.GetHashCode();
                if (this.SchoolId != null)
                    hashCode = hashCode * 59 + this.SchoolId.GetHashCode();
                if (this.SchoolYear != null)
                    hashCode = hashCode * 59 + this.SchoolYear.GetHashCode();
                if (this.Link != null)
                    hashCode = hashCode * 59 + this.Link.GetHashCode();
                return hashCode;
            }
        }
    }

}
