/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiContractedStaff
    /// </summary>
    [DataContract]
    public partial class EdFiContractedStaff :  IEquatable<EdFiContractedStaff>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiContractedStaff" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiContractedStaff() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiContractedStaff" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="asOfDate">The date of the reported contracted staff element. (required).</param>
        /// <param name="accountReference">accountReference (required).</param>
        /// <param name="staffReference">staffReference (required).</param>
        /// <param name="amountToDate">Current balance (amount paid to contractor) for account for the fiscal year. (required).</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiContractedStaff(string id = default(string), DateTime? asOfDate = default(DateTime?), EdFiAccountReference accountReference = default(EdFiAccountReference), EdFiStaffReference staffReference = default(EdFiStaffReference), double? amountToDate = default(double?), string etag = default(string))
        {
            // to ensure "asOfDate" is required (not null)
            if (asOfDate == null)
            {
                throw new InvalidDataException("asOfDate is a required property for EdFiContractedStaff and cannot be null");
            }
            else
            {
                this.AsOfDate = asOfDate;
            }
            // to ensure "accountReference" is required (not null)
            if (accountReference == null)
            {
                throw new InvalidDataException("accountReference is a required property for EdFiContractedStaff and cannot be null");
            }
            else
            {
                this.AccountReference = accountReference;
            }
            // to ensure "staffReference" is required (not null)
            if (staffReference == null)
            {
                throw new InvalidDataException("staffReference is a required property for EdFiContractedStaff and cannot be null");
            }
            else
            {
                this.StaffReference = staffReference;
            }
            // to ensure "amountToDate" is required (not null)
            if (amountToDate == null)
            {
                throw new InvalidDataException("amountToDate is a required property for EdFiContractedStaff and cannot be null");
            }
            else
            {
                this.AmountToDate = amountToDate;
            }
            this.Id = id;
            this.Etag = etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The date of the reported contracted staff element.
        /// </summary>
        /// <value>The date of the reported contracted staff element.</value>
        [DataMember(Name="asOfDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? AsOfDate { get; set; }

        /// <summary>
        /// Gets or Sets AccountReference
        /// </summary>
        [DataMember(Name="accountReference", EmitDefaultValue=false)]
        public EdFiAccountReference AccountReference { get; set; }

        /// <summary>
        /// Gets or Sets StaffReference
        /// </summary>
        [DataMember(Name="staffReference", EmitDefaultValue=false)]
        public EdFiStaffReference StaffReference { get; set; }

        /// <summary>
        /// Current balance (amount paid to contractor) for account for the fiscal year.
        /// </summary>
        /// <value>Current balance (amount paid to contractor) for account for the fiscal year.</value>
        [DataMember(Name="amountToDate", EmitDefaultValue=false)]
        public double? AmountToDate { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiContractedStaff {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  AsOfDate: ").Append(AsOfDate).Append("\n");
            sb.Append("  AccountReference: ").Append(AccountReference).Append("\n");
            sb.Append("  StaffReference: ").Append(StaffReference).Append("\n");
            sb.Append("  AmountToDate: ").Append(AmountToDate).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiContractedStaff);
        }

        /// <summary>
        /// Returns true if EdFiContractedStaff instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiContractedStaff to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiContractedStaff input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.AsOfDate == input.AsOfDate ||
                    (this.AsOfDate != null &&
                    this.AsOfDate.Equals(input.AsOfDate))
                ) && 
                (
                    this.AccountReference == input.AccountReference ||
                    (this.AccountReference != null &&
                    this.AccountReference.Equals(input.AccountReference))
                ) && 
                (
                    this.StaffReference == input.StaffReference ||
                    (this.StaffReference != null &&
                    this.StaffReference.Equals(input.StaffReference))
                ) && 
                (
                    this.AmountToDate == input.AmountToDate ||
                    (this.AmountToDate != null &&
                    this.AmountToDate.Equals(input.AmountToDate))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.AsOfDate != null)
                    hashCode = hashCode * 59 + this.AsOfDate.GetHashCode();
                if (this.AccountReference != null)
                    hashCode = hashCode * 59 + this.AccountReference.GetHashCode();
                if (this.StaffReference != null)
                    hashCode = hashCode * 59 + this.StaffReference.GetHashCode();
                if (this.AmountToDate != null)
                    hashCode = hashCode * 59 + this.AmountToDate.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }
    }

}
