/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiOpenStaffPosition
    /// </summary>
    [DataContract]
    public partial class EdFiOpenStaffPosition :  IEquatable<EdFiOpenStaffPosition>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiOpenStaffPosition" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiOpenStaffPosition() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiOpenStaffPosition" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="requisitionNumber">The number or identifier assigned to an open staff position, typically a requisition number assigned by Human Resources. (required).</param>
        /// <param name="educationOrganizationReference">educationOrganizationReference (required).</param>
        /// <param name="academicSubjects">An unordered collection of openStaffPositionAcademicSubjects. The teaching field required for the OpenStaffPosition, for example English/Language Arts, Reading, Mathematics, Science, Social Sciences, etc..</param>
        /// <param name="datePosted">Date the OpenStaffPosition was posted. (required).</param>
        /// <param name="datePostingRemoved">The date the posting was removed or filled..</param>
        /// <param name="employmentStatusDescriptor">Reflects the type of employment or contract desired for the position; for example:        Probationary        Contractual        Substitute/temporary        Tenured or permanent        Volunteer/no contract        ... (required).</param>
        /// <param name="instructionalGradeLevels">An unordered collection of openStaffPositionInstructionalGradeLevels. The set of grade levels for which the position&#39;s assignment is responsible..</param>
        /// <param name="positionTitle">The descriptive name of an individual&#39;s position..</param>
        /// <param name="postingResultDescriptor">Indication of whether the OpenStaffPosition was filled or retired without filling..</param>
        /// <param name="programAssignmentDescriptor">The name of the program for which the OpenStaffPosition will be assigned; for example:        Regular education        Title I-Academic        Title I-Non-Academic        Special Education&#39;        Bilingual/English as a Second Language..</param>
        /// <param name="staffClassificationDescriptor">The titles of employment, official status, or rank of education staff. (required).</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiOpenStaffPosition(string id = default(string), string requisitionNumber = default(string), EdFiEducationOrganizationReference educationOrganizationReference = default(EdFiEducationOrganizationReference), List<EdFiOpenStaffPositionAcademicSubject> academicSubjects = default(List<EdFiOpenStaffPositionAcademicSubject>), DateTime? datePosted = default(DateTime?), DateTime? datePostingRemoved = default(DateTime?), string employmentStatusDescriptor = default(string), List<EdFiOpenStaffPositionInstructionalGradeLevel> instructionalGradeLevels = default(List<EdFiOpenStaffPositionInstructionalGradeLevel>), string positionTitle = default(string), string postingResultDescriptor = default(string), string programAssignmentDescriptor = default(string), string staffClassificationDescriptor = default(string), string etag = default(string))
        {
            // to ensure "requisitionNumber" is required (not null)
            if (requisitionNumber == null)
            {
                throw new InvalidDataException("requisitionNumber is a required property for EdFiOpenStaffPosition and cannot be null");
            }
            else
            {
                this.RequisitionNumber = requisitionNumber;
            }
            // to ensure "educationOrganizationReference" is required (not null)
            if (educationOrganizationReference == null)
            {
                throw new InvalidDataException("educationOrganizationReference is a required property for EdFiOpenStaffPosition and cannot be null");
            }
            else
            {
                this.EducationOrganizationReference = educationOrganizationReference;
            }
            // to ensure "datePosted" is required (not null)
            if (datePosted == null)
            {
                throw new InvalidDataException("datePosted is a required property for EdFiOpenStaffPosition and cannot be null");
            }
            else
            {
                this.DatePosted = datePosted;
            }
            // to ensure "employmentStatusDescriptor" is required (not null)
            if (employmentStatusDescriptor == null)
            {
                throw new InvalidDataException("employmentStatusDescriptor is a required property for EdFiOpenStaffPosition and cannot be null");
            }
            else
            {
                this.EmploymentStatusDescriptor = employmentStatusDescriptor;
            }
            // to ensure "staffClassificationDescriptor" is required (not null)
            if (staffClassificationDescriptor == null)
            {
                throw new InvalidDataException("staffClassificationDescriptor is a required property for EdFiOpenStaffPosition and cannot be null");
            }
            else
            {
                this.StaffClassificationDescriptor = staffClassificationDescriptor;
            }
            this.Id = id;
            this.AcademicSubjects = academicSubjects;
            this.DatePostingRemoved = datePostingRemoved;
            this.InstructionalGradeLevels = instructionalGradeLevels;
            this.PositionTitle = positionTitle;
            this.PostingResultDescriptor = postingResultDescriptor;
            this.ProgramAssignmentDescriptor = programAssignmentDescriptor;
            this.Etag = etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The number or identifier assigned to an open staff position, typically a requisition number assigned by Human Resources.
        /// </summary>
        /// <value>The number or identifier assigned to an open staff position, typically a requisition number assigned by Human Resources.</value>
        [DataMember(Name="requisitionNumber", EmitDefaultValue=false)]
        public string RequisitionNumber { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name="educationOrganizationReference", EmitDefaultValue=false)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// An unordered collection of openStaffPositionAcademicSubjects. The teaching field required for the OpenStaffPosition, for example English/Language Arts, Reading, Mathematics, Science, Social Sciences, etc.
        /// </summary>
        /// <value>An unordered collection of openStaffPositionAcademicSubjects. The teaching field required for the OpenStaffPosition, for example English/Language Arts, Reading, Mathematics, Science, Social Sciences, etc.</value>
        [DataMember(Name="academicSubjects", EmitDefaultValue=false)]
        public List<EdFiOpenStaffPositionAcademicSubject> AcademicSubjects { get; set; }

        /// <summary>
        /// Date the OpenStaffPosition was posted.
        /// </summary>
        /// <value>Date the OpenStaffPosition was posted.</value>
        [DataMember(Name="datePosted", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? DatePosted { get; set; }

        /// <summary>
        /// The date the posting was removed or filled.
        /// </summary>
        /// <value>The date the posting was removed or filled.</value>
        [DataMember(Name="datePostingRemoved", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? DatePostingRemoved { get; set; }

        /// <summary>
        /// Reflects the type of employment or contract desired for the position; for example:        Probationary        Contractual        Substitute/temporary        Tenured or permanent        Volunteer/no contract        ...
        /// </summary>
        /// <value>Reflects the type of employment or contract desired for the position; for example:        Probationary        Contractual        Substitute/temporary        Tenured or permanent        Volunteer/no contract        ...</value>
        [DataMember(Name="employmentStatusDescriptor", EmitDefaultValue=false)]
        public string EmploymentStatusDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of openStaffPositionInstructionalGradeLevels. The set of grade levels for which the position&#39;s assignment is responsible.
        /// </summary>
        /// <value>An unordered collection of openStaffPositionInstructionalGradeLevels. The set of grade levels for which the position&#39;s assignment is responsible.</value>
        [DataMember(Name="instructionalGradeLevels", EmitDefaultValue=false)]
        public List<EdFiOpenStaffPositionInstructionalGradeLevel> InstructionalGradeLevels { get; set; }

        /// <summary>
        /// The descriptive name of an individual&#39;s position.
        /// </summary>
        /// <value>The descriptive name of an individual&#39;s position.</value>
        [DataMember(Name="positionTitle", EmitDefaultValue=false)]
        public string PositionTitle { get; set; }

        /// <summary>
        /// Indication of whether the OpenStaffPosition was filled or retired without filling.
        /// </summary>
        /// <value>Indication of whether the OpenStaffPosition was filled or retired without filling.</value>
        [DataMember(Name="postingResultDescriptor", EmitDefaultValue=false)]
        public string PostingResultDescriptor { get; set; }

        /// <summary>
        /// The name of the program for which the OpenStaffPosition will be assigned; for example:        Regular education        Title I-Academic        Title I-Non-Academic        Special Education&#39;        Bilingual/English as a Second Language.
        /// </summary>
        /// <value>The name of the program for which the OpenStaffPosition will be assigned; for example:        Regular education        Title I-Academic        Title I-Non-Academic        Special Education&#39;        Bilingual/English as a Second Language.</value>
        [DataMember(Name="programAssignmentDescriptor", EmitDefaultValue=false)]
        public string ProgramAssignmentDescriptor { get; set; }

        /// <summary>
        /// The titles of employment, official status, or rank of education staff.
        /// </summary>
        /// <value>The titles of employment, official status, or rank of education staff.</value>
        [DataMember(Name="staffClassificationDescriptor", EmitDefaultValue=false)]
        public string StaffClassificationDescriptor { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiOpenStaffPosition {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  RequisitionNumber: ").Append(RequisitionNumber).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  AcademicSubjects: ").Append(AcademicSubjects).Append("\n");
            sb.Append("  DatePosted: ").Append(DatePosted).Append("\n");
            sb.Append("  DatePostingRemoved: ").Append(DatePostingRemoved).Append("\n");
            sb.Append("  EmploymentStatusDescriptor: ").Append(EmploymentStatusDescriptor).Append("\n");
            sb.Append("  InstructionalGradeLevels: ").Append(InstructionalGradeLevels).Append("\n");
            sb.Append("  PositionTitle: ").Append(PositionTitle).Append("\n");
            sb.Append("  PostingResultDescriptor: ").Append(PostingResultDescriptor).Append("\n");
            sb.Append("  ProgramAssignmentDescriptor: ").Append(ProgramAssignmentDescriptor).Append("\n");
            sb.Append("  StaffClassificationDescriptor: ").Append(StaffClassificationDescriptor).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiOpenStaffPosition);
        }

        /// <summary>
        /// Returns true if EdFiOpenStaffPosition instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiOpenStaffPosition to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiOpenStaffPosition input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.RequisitionNumber == input.RequisitionNumber ||
                    (this.RequisitionNumber != null &&
                    this.RequisitionNumber.Equals(input.RequisitionNumber))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.AcademicSubjects == input.AcademicSubjects ||
                    this.AcademicSubjects != null &&
                    this.AcademicSubjects.SequenceEqual(input.AcademicSubjects)
                ) && 
                (
                    this.DatePosted == input.DatePosted ||
                    (this.DatePosted != null &&
                    this.DatePosted.Equals(input.DatePosted))
                ) && 
                (
                    this.DatePostingRemoved == input.DatePostingRemoved ||
                    (this.DatePostingRemoved != null &&
                    this.DatePostingRemoved.Equals(input.DatePostingRemoved))
                ) && 
                (
                    this.EmploymentStatusDescriptor == input.EmploymentStatusDescriptor ||
                    (this.EmploymentStatusDescriptor != null &&
                    this.EmploymentStatusDescriptor.Equals(input.EmploymentStatusDescriptor))
                ) && 
                (
                    this.InstructionalGradeLevels == input.InstructionalGradeLevels ||
                    this.InstructionalGradeLevels != null &&
                    this.InstructionalGradeLevels.SequenceEqual(input.InstructionalGradeLevels)
                ) && 
                (
                    this.PositionTitle == input.PositionTitle ||
                    (this.PositionTitle != null &&
                    this.PositionTitle.Equals(input.PositionTitle))
                ) && 
                (
                    this.PostingResultDescriptor == input.PostingResultDescriptor ||
                    (this.PostingResultDescriptor != null &&
                    this.PostingResultDescriptor.Equals(input.PostingResultDescriptor))
                ) && 
                (
                    this.ProgramAssignmentDescriptor == input.ProgramAssignmentDescriptor ||
                    (this.ProgramAssignmentDescriptor != null &&
                    this.ProgramAssignmentDescriptor.Equals(input.ProgramAssignmentDescriptor))
                ) && 
                (
                    this.StaffClassificationDescriptor == input.StaffClassificationDescriptor ||
                    (this.StaffClassificationDescriptor != null &&
                    this.StaffClassificationDescriptor.Equals(input.StaffClassificationDescriptor))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.RequisitionNumber != null)
                    hashCode = hashCode * 59 + this.RequisitionNumber.GetHashCode();
                if (this.EducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationReference.GetHashCode();
                if (this.AcademicSubjects != null)
                    hashCode = hashCode * 59 + this.AcademicSubjects.GetHashCode();
                if (this.DatePosted != null)
                    hashCode = hashCode * 59 + this.DatePosted.GetHashCode();
                if (this.DatePostingRemoved != null)
                    hashCode = hashCode * 59 + this.DatePostingRemoved.GetHashCode();
                if (this.EmploymentStatusDescriptor != null)
                    hashCode = hashCode * 59 + this.EmploymentStatusDescriptor.GetHashCode();
                if (this.InstructionalGradeLevels != null)
                    hashCode = hashCode * 59 + this.InstructionalGradeLevels.GetHashCode();
                if (this.PositionTitle != null)
                    hashCode = hashCode * 59 + this.PositionTitle.GetHashCode();
                if (this.PostingResultDescriptor != null)
                    hashCode = hashCode * 59 + this.PostingResultDescriptor.GetHashCode();
                if (this.ProgramAssignmentDescriptor != null)
                    hashCode = hashCode * 59 + this.ProgramAssignmentDescriptor.GetHashCode();
                if (this.StaffClassificationDescriptor != null)
                    hashCode = hashCode * 59 + this.StaffClassificationDescriptor.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }
    }

}
