/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiXFinanceLocalActual
    /// </summary>
    [DataContract]
    public partial class EdFiXFinanceLocalActual :  IEquatable<EdFiXFinanceLocalActual>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceLocalActual" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiXFinanceLocalActual() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceLocalActual" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="asOfDate">The date of the reported amount for Account. (required).</param>
        /// <param name="localAccountReference">localAccountReference (required).</param>
        /// <param name="amount">Current balance for the account. (required).</param>
        /// <param name="financialCollectionDescriptor">The accounting period or grouping for which the amount is collected..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiXFinanceLocalActual(string id = default(string), DateTime? asOfDate = default(DateTime?), EdFiXFinanceLocalAccountReference localAccountReference = default(EdFiXFinanceLocalAccountReference), double? amount = default(double?), string financialCollectionDescriptor = default(string), string etag = default(string))
        {
            // to ensure "asOfDate" is required (not null)
            if (asOfDate == null)
            {
                throw new InvalidDataException("asOfDate is a required property for EdFiXFinanceLocalActual and cannot be null");
            }
            else
            {
                this.AsOfDate = asOfDate;
            }
            // to ensure "localAccountReference" is required (not null)
            if (localAccountReference == null)
            {
                throw new InvalidDataException("localAccountReference is a required property for EdFiXFinanceLocalActual and cannot be null");
            }
            else
            {
                this.LocalAccountReference = localAccountReference;
            }
            // to ensure "amount" is required (not null)
            if (amount == null)
            {
                throw new InvalidDataException("amount is a required property for EdFiXFinanceLocalActual and cannot be null");
            }
            else
            {
                this.Amount = amount;
            }
            this.Id = id;
            this.FinancialCollectionDescriptor = financialCollectionDescriptor;
            this.Etag = etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The date of the reported amount for Account.
        /// </summary>
        /// <value>The date of the reported amount for Account.</value>
        [DataMember(Name="asOfDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? AsOfDate { get; set; }

        /// <summary>
        /// Gets or Sets LocalAccountReference
        /// </summary>
        [DataMember(Name="localAccountReference", EmitDefaultValue=false)]
        public EdFiXFinanceLocalAccountReference LocalAccountReference { get; set; }

        /// <summary>
        /// Current balance for the account.
        /// </summary>
        /// <value>Current balance for the account.</value>
        [DataMember(Name="amount", EmitDefaultValue=false)]
        public double? Amount { get; set; }

        /// <summary>
        /// The accounting period or grouping for which the amount is collected.
        /// </summary>
        /// <value>The accounting period or grouping for which the amount is collected.</value>
        [DataMember(Name="financialCollectionDescriptor", EmitDefaultValue=false)]
        public string FinancialCollectionDescriptor { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiXFinanceLocalActual {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  AsOfDate: ").Append(AsOfDate).Append("\n");
            sb.Append("  LocalAccountReference: ").Append(LocalAccountReference).Append("\n");
            sb.Append("  Amount: ").Append(Amount).Append("\n");
            sb.Append("  FinancialCollectionDescriptor: ").Append(FinancialCollectionDescriptor).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiXFinanceLocalActual);
        }

        /// <summary>
        /// Returns true if EdFiXFinanceLocalActual instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiXFinanceLocalActual to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiXFinanceLocalActual input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.AsOfDate == input.AsOfDate ||
                    (this.AsOfDate != null &&
                    this.AsOfDate.Equals(input.AsOfDate))
                ) && 
                (
                    this.LocalAccountReference == input.LocalAccountReference ||
                    (this.LocalAccountReference != null &&
                    this.LocalAccountReference.Equals(input.LocalAccountReference))
                ) && 
                (
                    this.Amount == input.Amount ||
                    (this.Amount != null &&
                    this.Amount.Equals(input.Amount))
                ) && 
                (
                    this.FinancialCollectionDescriptor == input.FinancialCollectionDescriptor ||
                    (this.FinancialCollectionDescriptor != null &&
                    this.FinancialCollectionDescriptor.Equals(input.FinancialCollectionDescriptor))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.AsOfDate != null)
                    hashCode = hashCode * 59 + this.AsOfDate.GetHashCode();
                if (this.LocalAccountReference != null)
                    hashCode = hashCode * 59 + this.LocalAccountReference.GetHashCode();
                if (this.Amount != null)
                    hashCode = hashCode * 59 + this.Amount.GetHashCode();
                if (this.FinancialCollectionDescriptor != null)
                    hashCode = hashCode * 59 + this.FinancialCollectionDescriptor.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }
    }

}
