/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiXFinanceProjectDimension
    /// </summary>
    [DataContract]
    public partial class EdFiXFinanceProjectDimension :  IEquatable<EdFiXFinanceProjectDimension>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceProjectDimension" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiXFinanceProjectDimension() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceProjectDimension" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="code">The code representation of the account Project dimension. (required).</param>
        /// <param name="fiscalYear">The fiscal year for which the account Project dimension is valid. (required).</param>
        /// <param name="codeName">A description of the account Project dimension..</param>
        /// <param name="reportingTags">An unordered collection of projectDimensionReportingTags. Optional tag for accountability reporting (e.g. ESSA)..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiXFinanceProjectDimension(string id = default(string), string code = default(string), int? fiscalYear = default(int?), string codeName = default(string), List<EdFiXFinanceProjectDimensionReportingTag> reportingTags = default(List<EdFiXFinanceProjectDimensionReportingTag>), string etag = default(string))
        {
            // to ensure "code" is required (not null)
            if (code == null)
            {
                throw new InvalidDataException("code is a required property for EdFiXFinanceProjectDimension and cannot be null");
            }
            else
            {
                this.Code = code;
            }
            // to ensure "fiscalYear" is required (not null)
            if (fiscalYear == null)
            {
                throw new InvalidDataException("fiscalYear is a required property for EdFiXFinanceProjectDimension and cannot be null");
            }
            else
            {
                this.FiscalYear = fiscalYear;
            }
            this.Id = id;
            this.CodeName = codeName;
            this.ReportingTags = reportingTags;
            this.Etag = etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The code representation of the account Project dimension.
        /// </summary>
        /// <value>The code representation of the account Project dimension.</value>
        [DataMember(Name="code", EmitDefaultValue=false)]
        public string Code { get; set; }

        /// <summary>
        /// The fiscal year for which the account Project dimension is valid.
        /// </summary>
        /// <value>The fiscal year for which the account Project dimension is valid.</value>
        [DataMember(Name="fiscalYear", EmitDefaultValue=false)]
        public int? FiscalYear { get; set; }

        /// <summary>
        /// A description of the account Project dimension.
        /// </summary>
        /// <value>A description of the account Project dimension.</value>
        [DataMember(Name="codeName", EmitDefaultValue=false)]
        public string CodeName { get; set; }

        /// <summary>
        /// An unordered collection of projectDimensionReportingTags. Optional tag for accountability reporting (e.g. ESSA).
        /// </summary>
        /// <value>An unordered collection of projectDimensionReportingTags. Optional tag for accountability reporting (e.g. ESSA).</value>
        [DataMember(Name="reportingTags", EmitDefaultValue=false)]
        public List<EdFiXFinanceProjectDimensionReportingTag> ReportingTags { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiXFinanceProjectDimension {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  Code: ").Append(Code).Append("\n");
            sb.Append("  FiscalYear: ").Append(FiscalYear).Append("\n");
            sb.Append("  CodeName: ").Append(CodeName).Append("\n");
            sb.Append("  ReportingTags: ").Append(ReportingTags).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiXFinanceProjectDimension);
        }

        /// <summary>
        /// Returns true if EdFiXFinanceProjectDimension instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiXFinanceProjectDimension to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiXFinanceProjectDimension input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.Code == input.Code ||
                    (this.Code != null &&
                    this.Code.Equals(input.Code))
                ) && 
                (
                    this.FiscalYear == input.FiscalYear ||
                    (this.FiscalYear != null &&
                    this.FiscalYear.Equals(input.FiscalYear))
                ) && 
                (
                    this.CodeName == input.CodeName ||
                    (this.CodeName != null &&
                    this.CodeName.Equals(input.CodeName))
                ) && 
                (
                    this.ReportingTags == input.ReportingTags ||
                    this.ReportingTags != null &&
                    this.ReportingTags.SequenceEqual(input.ReportingTags)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.Code != null)
                    hashCode = hashCode * 59 + this.Code.GetHashCode();
                if (this.FiscalYear != null)
                    hashCode = hashCode * 59 + this.FiscalYear.GetHashCode();
                if (this.CodeName != null)
                    hashCode = hashCode * 59 + this.CodeName.GetHashCode();
                if (this.ReportingTags != null)
                    hashCode = hashCode * 59 + this.ReportingTags.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }
    }

}
