/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// WiStudentTransportation
    /// </summary>
    [DataContract]
    public partial class WiStudentTransportation :  IEquatable<WiStudentTransportation>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WiStudentTransportation" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected WiStudentTransportation() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="WiStudentTransportation" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="transportationTypeDescriptor">Type of transportation aid (required).</param>
        /// <param name="educationOrganizationReference">educationOrganizationReference (required).</param>
        /// <param name="schoolYearTypeReference">schoolYearTypeReference (required).</param>
        /// <param name="studentReference">studentReference (required).</param>
        /// <param name="mileageCategory">Miles student transported. Ex: 1 miles, 15 miles, etc (required).</param>
        /// <param name="privateSchoolIndicator">Flag to indicate if student&#39;s primary school is a non-public school (required).</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public WiStudentTransportation(string id = default(string), string transportationTypeDescriptor = default(string), EdFiEducationOrganizationReference educationOrganizationReference = default(EdFiEducationOrganizationReference), EdFiSchoolYearTypeReference schoolYearTypeReference = default(EdFiSchoolYearTypeReference), EdFiStudentReference studentReference = default(EdFiStudentReference), int? mileageCategory = default(int?), bool? privateSchoolIndicator = default(bool?), string etag = default(string))
        {
            // to ensure "transportationTypeDescriptor" is required (not null)
            if (transportationTypeDescriptor == null)
            {
                throw new InvalidDataException("transportationTypeDescriptor is a required property for WiStudentTransportation and cannot be null");
            }
            else
            {
                this.TransportationTypeDescriptor = transportationTypeDescriptor;
            }
            // to ensure "educationOrganizationReference" is required (not null)
            if (educationOrganizationReference == null)
            {
                throw new InvalidDataException("educationOrganizationReference is a required property for WiStudentTransportation and cannot be null");
            }
            else
            {
                this.EducationOrganizationReference = educationOrganizationReference;
            }
            // to ensure "schoolYearTypeReference" is required (not null)
            if (schoolYearTypeReference == null)
            {
                throw new InvalidDataException("schoolYearTypeReference is a required property for WiStudentTransportation and cannot be null");
            }
            else
            {
                this.SchoolYearTypeReference = schoolYearTypeReference;
            }
            // to ensure "studentReference" is required (not null)
            if (studentReference == null)
            {
                throw new InvalidDataException("studentReference is a required property for WiStudentTransportation and cannot be null");
            }
            else
            {
                this.StudentReference = studentReference;
            }
            // to ensure "mileageCategory" is required (not null)
            if (mileageCategory == null)
            {
                throw new InvalidDataException("mileageCategory is a required property for WiStudentTransportation and cannot be null");
            }
            else
            {
                this.MileageCategory = mileageCategory;
            }
            // to ensure "privateSchoolIndicator" is required (not null)
            if (privateSchoolIndicator == null)
            {
                throw new InvalidDataException("privateSchoolIndicator is a required property for WiStudentTransportation and cannot be null");
            }
            else
            {
                this.PrivateSchoolIndicator = privateSchoolIndicator;
            }
            this.Id = id;
            this.Etag = etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// Type of transportation aid
        /// </summary>
        /// <value>Type of transportation aid</value>
        [DataMember(Name="transportationTypeDescriptor", EmitDefaultValue=false)]
        public string TransportationTypeDescriptor { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name="educationOrganizationReference", EmitDefaultValue=false)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// Gets or Sets SchoolYearTypeReference
        /// </summary>
        [DataMember(Name="schoolYearTypeReference", EmitDefaultValue=false)]
        public EdFiSchoolYearTypeReference SchoolYearTypeReference { get; set; }

        /// <summary>
        /// Gets or Sets StudentReference
        /// </summary>
        [DataMember(Name="studentReference", EmitDefaultValue=false)]
        public EdFiStudentReference StudentReference { get; set; }

        /// <summary>
        /// Miles student transported. Ex: 1 miles, 15 miles, etc
        /// </summary>
        /// <value>Miles student transported. Ex: 1 miles, 15 miles, etc</value>
        [DataMember(Name="mileageCategory", EmitDefaultValue=false)]
        public int? MileageCategory { get; set; }

        /// <summary>
        /// Flag to indicate if student&#39;s primary school is a non-public school
        /// </summary>
        /// <value>Flag to indicate if student&#39;s primary school is a non-public school</value>
        [DataMember(Name="privateSchoolIndicator", EmitDefaultValue=false)]
        public bool? PrivateSchoolIndicator { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class WiStudentTransportation {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  TransportationTypeDescriptor: ").Append(TransportationTypeDescriptor).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  SchoolYearTypeReference: ").Append(SchoolYearTypeReference).Append("\n");
            sb.Append("  StudentReference: ").Append(StudentReference).Append("\n");
            sb.Append("  MileageCategory: ").Append(MileageCategory).Append("\n");
            sb.Append("  PrivateSchoolIndicator: ").Append(PrivateSchoolIndicator).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as WiStudentTransportation);
        }

        /// <summary>
        /// Returns true if WiStudentTransportation instances are equal
        /// </summary>
        /// <param name="input">Instance of WiStudentTransportation to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(WiStudentTransportation input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.TransportationTypeDescriptor == input.TransportationTypeDescriptor ||
                    (this.TransportationTypeDescriptor != null &&
                    this.TransportationTypeDescriptor.Equals(input.TransportationTypeDescriptor))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.SchoolYearTypeReference == input.SchoolYearTypeReference ||
                    (this.SchoolYearTypeReference != null &&
                    this.SchoolYearTypeReference.Equals(input.SchoolYearTypeReference))
                ) && 
                (
                    this.StudentReference == input.StudentReference ||
                    (this.StudentReference != null &&
                    this.StudentReference.Equals(input.StudentReference))
                ) && 
                (
                    this.MileageCategory == input.MileageCategory ||
                    (this.MileageCategory != null &&
                    this.MileageCategory.Equals(input.MileageCategory))
                ) && 
                (
                    this.PrivateSchoolIndicator == input.PrivateSchoolIndicator ||
                    (this.PrivateSchoolIndicator != null &&
                    this.PrivateSchoolIndicator.Equals(input.PrivateSchoolIndicator))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.TransportationTypeDescriptor != null)
                    hashCode = hashCode * 59 + this.TransportationTypeDescriptor.GetHashCode();
                if (this.EducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationReference.GetHashCode();
                if (this.SchoolYearTypeReference != null)
                    hashCode = hashCode * 59 + this.SchoolYearTypeReference.GetHashCode();
                if (this.StudentReference != null)
                    hashCode = hashCode * 59 + this.StudentReference.GetHashCode();
                if (this.MileageCategory != null)
                    hashCode = hashCode * 59 + this.MileageCategory.GetHashCode();
                if (this.PrivateSchoolIndicator != null)
                    hashCode = hashCode * 59 + this.PrivateSchoolIndicator.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }
    }

}
